using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using log4net;
using vpnsim.authenticator;
using vpnsim.authenticator.core;

namespace vpnsim.authenticator.core
{
    class main
    {
        /// <summary>
        /// Creates an instance of the authenttication server
        /// </summary>
        static void Main()
        {
            AuthenticationServer authServer = new AuthenticationServer();
         //   authServer.AuthenticateUser("242023800085700", new byte[] { 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01});
        }

    }

    public class AuthenticationServer
    {
        private static readonly ILog log = LogManager.GetLogger(typeof(main));
        private ArrayList authenticatedUsers = new ArrayList();
        /// <summary>
        /// Constructor for the main class of this package. Creates instances of Communicator and QueryListener, and starts listening for incoming authentication requests.
        /// </summary>
        public AuthenticationServer()
        {

            Communicator communicator = new Communicator(this);
            QueryListener queryListener = new QueryListener(this);
            log4net.Config.BasicConfigurator.Configure();

            if (communicator.Start() == true)
            {
                log.Info("UDP Server Running on port " + communicator.udpPort);
            }
            if (queryListener.Start() == true)
            {
                log.Info("QueryListener Running on port " + queryListener.udpPort);
            }
        }

        /// <summary>
        /// Method for telling the authentication server when a new user has been authenticated. Inserts new users into the database and updates existing users.
        /// </summary>
        /// <param name="username">The username that is authenticated</param>
        /// <param name="key">The authentication key for that user</param>
        public void AuthenticateUser(string username, byte[] key)
        {
            foreach (User user in authenticatedUsers)
            {
                if (user.username.Equals(username))
                {
                    user.key = key;
                    log.Info("Key updated for user "+username);
                    return;
                }
            }
            authenticatedUsers.Add(new User(username, key));
            log.Info("User " + username + " added to database");
        }

        /// <summary>
        /// Method for fetching the key for a given user. If the user doesn't exists, returns false.
        /// </summary>
        /// <param name="username">The username to look up</param>
        /// <param name="key">The return value of the key</param>
        /// <returns>Returns whether the user is authenticated or not</returns>
        public bool GetKey(string username, out byte[] key)
        {
            foreach (User user in authenticatedUsers)
            {
                if (user.username.Equals(username))
                {
                    key = user.key;
                    return true;
                }
            }
            key = null;
            return false;
        }
    }
    /// <summary>
    /// Internal class for representing authenticated users
    /// </summary>
    internal class User
    {
        public string username;
        public byte[] key;

        /// <summary>
        /// Create a new user object
        /// </summary>
        /// <param name="username">Username of the user</param>
        /// <param name="key">Authentication key of the user</param>
        internal User(string username, byte[] key)
        {
            this.username = username;
            this.key = key;
        }
    }
}