using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using eap;
using log4net;
using vpnsim.authenticator.core;

namespace vpnsim.authenticator.core
{
    /// <summary>
    /// An authentication listener. Listens for incoming EAP requests
    /// </summary>
    public class Communicator
    {
        private static readonly ILog log = LogManager.GetLogger(typeof(Communicator));
        public Thread udpThread;
        public ushort udpPort = 5746;
        private EAPHandler eapHandler;
        private SimFunctions simFunctions;

        /// <summary>
        /// Constructor for the communicator object
        /// </summary>
        /// <param name="authServer">Reference to the authentication server object this communicator runs on</param>
        public Communicator(AuthenticationServer authServer)
        {
            simFunctions = new SimFunctions(authServer);
            eapHandler = new EAPHandler(simFunctions);
            
        }
        
        /// <summary>
        /// Starts running the EAP Communication listener
        /// </summary>
        /// <returns>Returns whether the communicator was able to start</returns>
        public bool Start()
        {
            try
            {
                udpThread = new Thread(new ThreadStart(StartReceive));
                udpThread.Start();
                return true;
            }catch (Exception)
            {
                udpThread.Abort();
                return false;
            }
        }
        private void StartReceive()
        {
            IPHostEntry localHostEntry;
            Socket soUdp = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            try
            {
                localHostEntry = Dns.GetHostEntry(Dns.GetHostName());
            }catch(Exception)
            {
                log.Error("Local host could not be found");
                return;
            }
            IPEndPoint localIpEndPoint = new IPEndPoint(localHostEntry.AddressList[0], udpPort);
            soUdp.Bind(localIpEndPoint);
            while(true)
            {
                byte[] received = new byte[255]; 
                IPEndPoint tmpIpEndPoint = new IPEndPoint(localHostEntry.AddressList[0], udpPort);
                EndPoint remoteEP = (tmpIpEndPoint);
                //int bytesReceived = soUdp.ReceiveFrom(received, 4, SocketFlags.Peek, ref remoteEP);
                int bytesReceived = soUdp.ReceiveFrom(received, ref remoteEP);
                Array.Resize(ref received, bytesReceived);
                EAPMessage message = EAPMessage.ParseBytes(received);
                log.Info("Received " + message.ToString());
                EAPMessage response = eapHandler.HandleMessage(message);
                log.Info("Sending " + response.ToString());
                soUdp.SendTo(response.GetBytes(), remoteEP);
            }
        }
    }

}
