using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using log4net;

namespace vpnsim.authenticator.core
{
    /// <summary>
    /// Very basic class for listening for incoming status queries
    /// </summary>
    public class QueryListener
    {
        private static readonly ILog log = LogManager.GetLogger(typeof(QueryListener));
        public Thread udpThread;
        public int udpPort = 5747;
        private AuthenticationServer authServer;

        /// <summary>
        /// Creates an instance of the querylistener object
        /// </summary>
        /// <param name="myAs">The authentication server object this querylistener is running on</param>
        public QueryListener(AuthenticationServer myAs)
        {
            this.authServer = myAs;
        }
        
        /// <summary>
        /// Starts running the querylistener
        /// </summary>
        /// <returns></returns>
        public bool Start()
        {
            try
            {
                udpThread = new Thread(new ThreadStart(StartReceive));
                udpThread.Start();
                return true;
            }catch (Exception)
            {
                udpThread.Abort();
                return false;
            }
        }

        /// <summary>
        /// Starts running the listen loop. Just listens for udp packets containing a username, and returns the associated key
        /// </summary>
        private void StartReceive()
        {
            IPHostEntry localHostEntry;
            Socket soUdp = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            try
            {
                localHostEntry = Dns.GetHostEntry(Dns.GetHostName());
            }catch(Exception)
            {
                log.Error("Local host could not be found");
                return;
            }
            IPEndPoint localIpEndPoint = new IPEndPoint(localHostEntry.AddressList[0], udpPort);
            soUdp.Bind(localIpEndPoint);
            while(true)
            {
                
                byte[] received = new byte[255]; 
                IPEndPoint tmpIpEndPoint = new IPEndPoint(localHostEntry.AddressList[0], udpPort);
                EndPoint remoteEP = (tmpIpEndPoint);
                //int bytesReceived = soUdp.ReceiveFrom(received, 4, SocketFlags.Peek, ref remoteEP);
                int bytesReceived = soUdp.ReceiveFrom(received, ref remoteEP);
                Array.Resize(ref received, bytesReceived);
                string userId = System.Text.Encoding.ASCII.GetString(received);
                byte[] key;
                if (!authServer.GetKey(userId, out key))
                {
                    log.Debug("Could not find user, sending blank key");
                    key = new byte[] { 0x00 };
                }
                else
                {
                    log.Debug("Received authentication query for user " + userId);
                }

                soUdp.SendTo(key, remoteEP);
            }
        }
    }

}
