using System;
using System.Collections.Generic;
using System.Text;

namespace eap
{
    /// <summary>
    /// An interface for the EAPHandler class. Performs different SIM-related functions for the EAP-SIM-authentication
    /// procedure.
    /// </summary>
    public interface ISimFunctions
    {
        /// <summary>
        /// After exchaning keys, register a new key with the given user
        /// </summary>
        /// <param name="imsi">The imsi</param>
        /// <param name="key">The key associated with the imsi</param>
        void RegisterKey(byte[] imsi, byte[] key);

        /// <summary>
        /// Validates an IMSI
        /// </summary>
        /// <param name="imsi">The IMSI to validate</param>
        /// <returns>Whether the IMSI is valid</returns>
        bool ValidateIMSI(byte[] imsi);

        /// <summary>
        /// Get a number of triplets for a given IMSI
        /// </summary>
        /// <param name="imsi">The IMSI to look up</param>
        /// <param name="i">The number of triplets to return</param>
        /// <param name="sres">SRES value of the triplet(s, concatenated)</param>
        /// <param name="Kc">Kc value of the triplet(s, concatenated)</param>
        /// <returns></returns>
        byte[] GetRANDs(byte[] imsi, int i, out byte[] sres, out byte[] Kc);
        bool CalculateSRES(byte[] rands, out byte[] sres, out byte[] Kc);
        byte[] GetMasterKey(byte[] imsi, byte[] kc, byte[] nonce, short[] verList, short selectedVersion);
    }
}
