using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using eap;
using log4net;

namespace vpnsim.supplicant.applicationSupplicant.eapCommunicator
{
    /// <summary>
    /// A class for communicating with an EAP Server
    /// </summary>
    public class EapClient
    {
        private static readonly ILog log = LogManager.GetLogger(typeof(EapClient));
        private IPEndPoint authenticatorAddress;
        
        /// <summary>
        /// Constructor for the EAP Client
        /// </summary>
        /// <param name="authenticator">Address of the authenticator to connect to</param>
        public EapClient(IPEndPoint authenticator)
        {
            this.authenticatorAddress = authenticator;
            
        }

        /// <summary>
        /// Send an EAP message to the authenticator
        /// </summary>
        /// <param name="message">The message to send</param>
        /// <returns>The response</returns>
        public EAPMessage Send(EAPMessage message)
        {
            UdpClient udpClient = new UdpClient();
            //IPHostEntry remoteHostEntry = Dns.GetHostEntry(serverName);
            //IPEndPoint remoteIpEndPoint = new IPEndPoint(remoteHostEntry.AddressList[0], udpPort);
            udpClient.Connect(authenticatorAddress);
     
            int nBytesSent = udpClient.Send(message.GetBytes(), message.Length());    

            
            Byte[] received;// = new Byte[512];
            try{
                received = udpClient.Receive(ref authenticatorAddress);
            }
            catch (Exception)
            {
                log.Debug("Error connecting to authentication server");
                return null;
            }
            udpClient.Close();
            //String dataReceived = System.Text.Encoding.ASCII.GetString(received);
            //MessageBox.Show(dataReceived);
            //udpClient.Send(message.GetBytes(), message.Length());
            EAPMessage response = EAPMessage.ParseBytes(received);
            return response;
        }
    }
}