using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using log4net;
using vpnsim.supplicant;
using SCLib;
using vpnsim.supplicant.applicationSupplicant.core;
using vpnsim.supplicant.simSupplicant;
using vpnsim.supplicant.simSupplicant.bluetooth;
using vpnsim.supplicant.simSupplicant.smartCard;
//required for using external .DLLs

namespace vpnsim.supplicant.applicationSupplicant.userInterface
{
    /// <summary>
    /// The main settings form
    /// </summary>
    public class SupplicantForm : Form
    {
        private static readonly ILog log = LogManager.GetLogger(typeof(SupplicantForm));
        private DebugLog logWindow;
        private Supplicant supplicant;
        private MenuItem menuItem6;
        private MenuItem menuItem7;
        private RadioButton rbDummy;
        private bool authenticated = false;
        
        public SupplicantForm(DebugLog logwindow)
        {
            InitializeComponent();
            this.logWindow = logwindow;

            this.Closing += new CancelEventHandler(SupplicantForm_Closing);
            RefreshFields();

        
        }

        /// <summary>
        /// Set the parent supplicant
        /// </summary>
        /// <param name="supplicant"></param>
        public void SetSupplicant(Supplicant supplicant)
        {
            this.supplicant = supplicant;
        }
        #region Windows Form Designer generated code

        private NotifyIcon s_TrayIcon;
        private ContextMenu ctxMenu;
        private IContainer components;
        private MenuItem menuItem1;
        private MenuItem menuItem2;
        private MenuItem menuItem3;
        private MenuItem menuItem4;
        private MenuItem menuItem5;
        private GroupBox gbAuthSupplicant;
        private Label lblIpAddress;
        private TextBox txtAuthSupplicantAddress;
        private GroupBox gbSim;
        private Button cmdSelectDevice;
        private RadioButton rbBluetooth;
        private RadioButton rbSmartCardReader;
        private Label lblSimCommunication;
        private TextBox txtPinCode;
        private Label lblPort;
        private Button cmdOK;
        private Button cmdCancel;
        private TextBox txtAuthSupplicantPort;
        private Label lblPinCode;

        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(SupplicantForm));
            this.s_TrayIcon = new System.Windows.Forms.NotifyIcon(this.components);
            this.ctxMenu = new System.Windows.Forms.ContextMenu();
            this.menuItem1 = new System.Windows.Forms.MenuItem();
            this.menuItem2 = new System.Windows.Forms.MenuItem();
            this.menuItem6 = new System.Windows.Forms.MenuItem();
            this.menuItem5 = new System.Windows.Forms.MenuItem();
            this.menuItem7 = new System.Windows.Forms.MenuItem();
            this.menuItem3 = new System.Windows.Forms.MenuItem();
            this.menuItem4 = new System.Windows.Forms.MenuItem();
            this.gbAuthSupplicant = new System.Windows.Forms.GroupBox();
            this.txtAuthSupplicantPort = new System.Windows.Forms.TextBox();
            this.lblPort = new System.Windows.Forms.Label();
            this.txtAuthSupplicantAddress = new System.Windows.Forms.TextBox();
            this.lblIpAddress = new System.Windows.Forms.Label();
            this.gbSim = new System.Windows.Forms.GroupBox();
            this.cmdSelectDevice = new System.Windows.Forms.Button();
            this.rbBluetooth = new System.Windows.Forms.RadioButton();
            this.rbSmartCardReader = new System.Windows.Forms.RadioButton();
            this.lblSimCommunication = new System.Windows.Forms.Label();
            this.lblPinCode = new System.Windows.Forms.Label();
            this.txtPinCode = new System.Windows.Forms.TextBox();
            this.cmdOK = new System.Windows.Forms.Button();
            this.cmdCancel = new System.Windows.Forms.Button();
            this.rbDummy = new System.Windows.Forms.RadioButton();
            this.gbAuthSupplicant.SuspendLayout();
            this.gbSim.SuspendLayout();
            this.SuspendLayout();
            // 
            // s_TrayIcon
            // 
            this.s_TrayIcon.ContextMenu = this.ctxMenu;
            this.s_TrayIcon.Icon = global::vpnsim.supplicant.Properties.Resources.client;
            this.s_TrayIcon.Text = "Tool Tip";
            this.s_TrayIcon.Visible = true;
            this.s_TrayIcon.DoubleClick += new System.EventHandler(this.s_TrayIcon_DoubleClick);
            // 
            // ctxMenu
            // 
            this.ctxMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem1,
            this.menuItem2,
            this.menuItem6,
            this.menuItem3,
            this.menuItem4});
            // 
            // menuItem1
            // 
            this.menuItem1.Index = 0;
            this.menuItem1.Text = "Settings";
            this.menuItem1.Click += new System.EventHandler(this.OpenWindow);
            // 
            // menuItem2
            // 
            this.menuItem2.Index = 1;
            this.menuItem2.Text = "Log Window";
            this.menuItem2.Click += new System.EventHandler(this.DisplayLogWindow);
            // 
            // menuItem6
            // 
            this.menuItem6.Index = 2;
            this.menuItem6.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem5,
            this.menuItem7});
            this.menuItem6.Text = "Debug";
            // 
            // menuItem5
            // 
            this.menuItem5.Index = 0;
            this.menuItem5.Text = "Authenticate";
            this.menuItem5.Click += new System.EventHandler(this.Authenticate);
            // 
            // menuItem7
            // 
            this.menuItem7.Index = 1;
            this.menuItem7.Text = "GenerateTriplets";
            this.menuItem7.Click += new System.EventHandler(this.GenerateTriplets);
            // 
            // menuItem3
            // 
            this.menuItem3.Index = 3;
            this.menuItem3.Text = "-";
            // 
            // menuItem4
            // 
            this.menuItem4.Index = 4;
            this.menuItem4.Text = "Exit";
            this.menuItem4.Click += new System.EventHandler(this.CloseApplication);
            // 
            // gbAuthSupplicant
            // 
            this.gbAuthSupplicant.Controls.Add(this.txtAuthSupplicantPort);
            this.gbAuthSupplicant.Controls.Add(this.lblPort);
            this.gbAuthSupplicant.Controls.Add(this.txtAuthSupplicantAddress);
            this.gbAuthSupplicant.Controls.Add(this.lblIpAddress);
            this.gbAuthSupplicant.Location = new System.Drawing.Point(12, 12);
            this.gbAuthSupplicant.Name = "gbAuthSupplicant";
            this.gbAuthSupplicant.Size = new System.Drawing.Size(270, 54);
            this.gbAuthSupplicant.TabIndex = 11;
            this.gbAuthSupplicant.TabStop = false;
            this.gbAuthSupplicant.Text = "Authentication Supplicant";
            // 
            // txtAuthSupplicantPort
            // 
            this.txtAuthSupplicantPort.Location = new System.Drawing.Point(215, 19);
            this.txtAuthSupplicantPort.Name = "txtAuthSupplicantPort";
            this.txtAuthSupplicantPort.Size = new System.Drawing.Size(37, 20);
            this.txtAuthSupplicantPort.TabIndex = 3;
            this.txtAuthSupplicantPort.TextChanged += new System.EventHandler(this.inputChanged);
            // 
            // lblPort
            // 
            this.lblPort.AutoSize = true;
            this.lblPort.Location = new System.Drawing.Point(180, 22);
            this.lblPort.Name = "lblPort";
            this.lblPort.Size = new System.Drawing.Size(29, 13);
            this.lblPort.TabIndex = 2;
            this.lblPort.Text = "Port:";
            // 
            // txtAuthSupplicantAddress
            // 
            this.txtAuthSupplicantAddress.Location = new System.Drawing.Point(74, 19);
            this.txtAuthSupplicantAddress.Name = "txtAuthSupplicantAddress";
            this.txtAuthSupplicantAddress.Size = new System.Drawing.Size(100, 20);
            this.txtAuthSupplicantAddress.TabIndex = 1;
            this.txtAuthSupplicantAddress.TextChanged += new System.EventHandler(this.inputChanged);
            // 
            // lblIpAddress
            // 
            this.lblIpAddress.AutoSize = true;
            this.lblIpAddress.Location = new System.Drawing.Point(7, 22);
            this.lblIpAddress.Name = "lblIpAddress";
            this.lblIpAddress.Size = new System.Drawing.Size(61, 13);
            this.lblIpAddress.TabIndex = 0;
            this.lblIpAddress.Text = "IP Address:";
            // 
            // gbSim
            // 
            this.gbSim.Controls.Add(this.cmdSelectDevice);
            this.gbSim.Controls.Add(this.rbDummy);
            this.gbSim.Controls.Add(this.rbBluetooth);
            this.gbSim.Controls.Add(this.rbSmartCardReader);
            this.gbSim.Controls.Add(this.lblSimCommunication);
            this.gbSim.Controls.Add(this.lblPinCode);
            this.gbSim.Controls.Add(this.txtPinCode);
            this.gbSim.Location = new System.Drawing.Point(12, 72);
            this.gbSim.Name = "gbSim";
            this.gbSim.Size = new System.Drawing.Size(270, 141);
            this.gbSim.TabIndex = 12;
            this.gbSim.TabStop = false;
            this.gbSim.Text = "SIM Card Settings";
            // 
            // cmdSelectDevice
            // 
            this.cmdSelectDevice.Location = new System.Drawing.Point(156, 75);
            this.cmdSelectDevice.Name = "cmdSelectDevice";
            this.cmdSelectDevice.Size = new System.Drawing.Size(96, 23);
            this.cmdSelectDevice.TabIndex = 5;
            this.cmdSelectDevice.Text = "Select &Device..";
            this.cmdSelectDevice.UseVisualStyleBackColor = true;
            this.cmdSelectDevice.Click += new System.EventHandler(this.cmdSelectDevice_Click);
            // 
            // rbBluetooth
            // 
            this.rbBluetooth.AutoSize = true;
            this.rbBluetooth.Location = new System.Drawing.Point(28, 86);
            this.rbBluetooth.Name = "rbBluetooth";
            this.rbBluetooth.Size = new System.Drawing.Size(70, 17);
            this.rbBluetooth.TabIndex = 4;
            this.rbBluetooth.TabStop = true;
            this.rbBluetooth.Text = "Bluetooth";
            this.rbBluetooth.UseVisualStyleBackColor = true;
            this.rbBluetooth.CheckedChanged += new System.EventHandler(this.inputChanged);
            // 
            // rbSmartCardReader
            // 
            this.rbSmartCardReader.AutoSize = true;
            this.rbSmartCardReader.Location = new System.Drawing.Point(28, 69);
            this.rbSmartCardReader.Name = "rbSmartCardReader";
            this.rbSmartCardReader.Size = new System.Drawing.Size(115, 17);
            this.rbSmartCardReader.TabIndex = 3;
            this.rbSmartCardReader.TabStop = true;
            this.rbSmartCardReader.Text = "Smart Card Reader";
            this.rbSmartCardReader.UseVisualStyleBackColor = true;
            this.rbSmartCardReader.CheckedChanged += new System.EventHandler(this.inputChanged);
            // 
            // lblSimCommunication
            // 
            this.lblSimCommunication.AutoSize = true;
            this.lblSimCommunication.Location = new System.Drawing.Point(7, 53);
            this.lblSimCommunication.Name = "lblSimCommunication";
            this.lblSimCommunication.Size = new System.Drawing.Size(124, 13);
            this.lblSimCommunication.TabIndex = 2;
            this.lblSimCommunication.Text = "Sim Card Communication";
            // 
            // lblPinCode
            // 
            this.lblPinCode.AutoSize = true;
            this.lblPinCode.Location = new System.Drawing.Point(7, 22);
            this.lblPinCode.Name = "lblPinCode";
            this.lblPinCode.Size = new System.Drawing.Size(56, 13);
            this.lblPinCode.TabIndex = 0;
            this.lblPinCode.Text = "PIN Code:";
            // 
            // txtPinCode
            // 
            this.txtPinCode.Location = new System.Drawing.Point(74, 19);
            this.txtPinCode.Name = "txtPinCode";
            this.txtPinCode.Size = new System.Drawing.Size(51, 20);
            this.txtPinCode.TabIndex = 1;
            this.txtPinCode.TextChanged += new System.EventHandler(this.inputChanged);
            // 
            // cmdOK
            // 
            this.cmdOK.Location = new System.Drawing.Point(127, 219);
            this.cmdOK.Name = "cmdOK";
            this.cmdOK.Size = new System.Drawing.Size(75, 23);
            this.cmdOK.TabIndex = 13;
            this.cmdOK.Text = "&Ok";
            this.cmdOK.UseVisualStyleBackColor = true;
            this.cmdOK.Click += new System.EventHandler(this.cmdOK_Click);
            // 
            // cmdCancel
            // 
            this.cmdCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.cmdCancel.Location = new System.Drawing.Point(208, 219);
            this.cmdCancel.Name = "cmdCancel";
            this.cmdCancel.Size = new System.Drawing.Size(75, 23);
            this.cmdCancel.TabIndex = 14;
            this.cmdCancel.Text = "&Cancel";
            this.cmdCancel.UseVisualStyleBackColor = true;
            this.cmdCancel.Click += new System.EventHandler(this.cmdCancel_Click);
            // 
            // rbDummy
            // 
            this.rbDummy.AutoSize = true;
            this.rbDummy.Location = new System.Drawing.Point(28, 103);
            this.rbDummy.Name = "rbDummy";
            this.rbDummy.Size = new System.Drawing.Size(105, 17);
            this.rbDummy.TabIndex = 4;
            this.rbDummy.TabStop = true;
            this.rbDummy.Text = "Dummy Interface";
            this.rbDummy.UseVisualStyleBackColor = true;
            this.rbDummy.CheckedChanged += new System.EventHandler(this.inputChanged);
            // 
            // SupplicantForm
            // 
            this.ClientSize = new System.Drawing.Size(295, 254);
            this.Controls.Add(this.cmdCancel);
            this.Controls.Add(this.cmdOK);
            this.Controls.Add(this.gbSim);
            this.Controls.Add(this.gbAuthSupplicant);
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.MaximizeBox = false;
            this.MinimizeBox = false;
            this.Name = "SupplicantForm";
            this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
            this.Text = "Supplicant Settings";
            this.gbAuthSupplicant.ResumeLayout(false);
            this.gbAuthSupplicant.PerformLayout();
            this.gbSim.ResumeLayout(false);
            this.gbSim.PerformLayout();
            this.ResumeLayout(false);

        }

        private void OpenWindow(object sender, EventArgs e)
        {
            log.Debug("Displaying main window.");
            this.Visible = true;
        }
        private void DisplayLogWindow(object sender, EventArgs e)
        {
            logWindow.Visible = true;
        }
        private void Authenticate(object sender, EventArgs e)
        {
            supplicant.Authenticate();
        }

        private void CloseApplication(object sender, EventArgs e)
        {
            this.s_TrayIcon.Visible = false;
            this.Dispose();
            logWindow.Shutdown();
            supplicant.Exit();
        }

        #endregion
        
        /// <summary>
        /// Changes the display status of the settings window. Hides/UNhides
        /// </summary>
        /// <param name="show"></param>
        public void DisplayWindow(bool show)
        {
            if (show.Equals(this.Visible)) return;

            this.Visible = show;
            this.ShowInTaskbar = show;
        }

        private void SupplicantForm_Closing(object sender, CancelEventArgs e)
        {
            DisplayWindow(false);
            e.Cancel = true;
        }
  

        private void inputChanged(object sender, EventArgs e)
        {
            cmdSelectDevice.Enabled = !rbDummy.Checked;
        }
        private void s_TrayIcon_DoubleClick(object sender, EventArgs e)
        {
            DisplayWindow(true);
        }

        private void cmdOK_Click(object sender, EventArgs e)
        {
            DisplayWindow(false);
            //Make changes to running application
            bool authResult = supplicant.ChangeAuthSupplicant(txtAuthSupplicantAddress.Text, txtAuthSupplicantPort.Text);
            bool pinResult = supplicant.ChangePinCode(txtPinCode.Text);
            if (!authResult)
            {
                log.Error("Authsupplicant address/port invalid.");
                MessageBox.Show("Verify the Authentication Supplicant address", "Error parsing address",
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                DisplayWindow(true);
                txtAuthSupplicantAddress.Focus();
                return;
            }
            else if (!pinResult)
            {
                log.Error("Pincode format invalid.");
                MessageBox.Show("Verify the format of the Pin Code", "Error setting Pin Code", MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
                DisplayWindow(true);
                txtPinCode.Focus();
                return;
            }
            bool SmartCardOK = (rbSmartCardReader.Checked&&!supplicant.GetSimSupplicant().scInterfaceName.Equals(""));
            bool BluetoothOK = (rbBluetooth.Checked&&!supplicant.GetSimSupplicant().btInterfaceAddress.Equals(""));
            //Is a valid interface selected?
            if (!(rbDummy.Checked || SmartCardOK || BluetoothOK))
            {
                MessageBox.Show("Please choose a SIM interface", "No SIM interface", MessageBoxButtons.OK,
                                MessageBoxIcon.Information);
                DisplayWindow(true);
                return;
            }

            if (rbSmartCardReader.Checked) supplicant.ChangeInterface(0x00);
            else if (rbBluetooth.Checked) supplicant.ChangeInterface(0x01);
            else if (rbDummy.Checked) supplicant.ChangeInterface(0x02);

            supplicant.SaveSettings();


            if (this.rbSmartCardReader.Checked) Properties.Settings.Default.simCommunication = 0x00;
            else if (this.rbBluetooth.Checked) Properties.Settings.Default.simCommunication = 0x01;
            else if (this.rbDummy.Checked) Properties.Settings.Default.simCommunication = 0x02;

            supplicant.Ready();
        }
        private void RefreshFields()
        {
            txtAuthSupplicantAddress.Text = Properties.Settings.Default.authSupplicantAddress;
            txtAuthSupplicantPort.Text = Properties.Settings.Default.authSupplicantPort.ToString();
            txtPinCode.Text = Properties.Settings.Default.pinCode;
            rbSmartCardReader.Enabled = SmartCardInterface.IsAvailable();
            rbBluetooth.Enabled = BluetoothInterface.IsAvailable();

            //Both Readers Available
            if(rbSmartCardReader.Enabled && rbBluetooth.Enabled)
            {
                rbSmartCardReader.Checked = false;
                rbBluetooth.Checked = false;
                rbDummy.Checked = false;

                if(Properties.Settings.Default.simCommunication.Equals(0x00))
                {
                    rbSmartCardReader.Checked=true;
                }else if(Properties.Settings.Default.simCommunication.Equals(0x01))
                {
                    rbBluetooth.Checked = true;
                }else{
                    rbDummy.Checked = true;
                }
            }else
            {
                rbSmartCardReader.Checked = rbSmartCardReader.Enabled;
                rbBluetooth.Checked = rbBluetooth.Enabled;

                if(!(rbSmartCardReader.Enabled&&rbBluetooth.Enabled)) //None enabled
                {
                    rbDummy.Checked = true;
                }
            }
            cmdSelectDevice.Enabled = !rbDummy.Checked;
        }

        private void cmdCancel_Click(object sender, EventArgs e)
        {
            DisplayWindow(false);
            RefreshFields();
            if (!supplicant.running) CloseApplication(null, null);
        }

        /// <summary>
        /// Change the tray icon to authenticated status
        /// </summary>
        /// <param name="authenticated"></param>
        public void setAuthenticated(bool authenticated)
        {
            if(authenticated)
            {
                this.s_TrayIcon.Icon = Properties.Resources.authenticated;
                authenticated = true;
            }else
            {
                this.s_TrayIcon.Icon = Properties.Resources.client;
                authenticated = false;
            }
        }

        /// <summary>
        /// Change the tray icon to connected status
        /// </summary>
        /// <param name="connected"></param>
        public void setConnected(bool connected)
        {
            if (connected)
            {
                this.s_TrayIcon.Icon = Properties.Resources.link;
            }else if(authenticated)
            {
                this.s_TrayIcon.Icon = Properties.Resources.authenticated;
            }else
            {
                this.s_TrayIcon.Icon = Properties.Resources.client;
            }
        }
        private void cmdSelectDevice_Click(object sender, EventArgs e)
        {
            try
            {
                if (rbBluetooth.Checked)
                {
                    supplicant.GetSimSupplicant().btInterfaceAddress = BluetoothInterface.SelectInterface();
                }
                else if (rbSmartCardReader.Checked)
                {
                    supplicant.GetSimSupplicant().scInterfaceName = SmartCardInterface.SelectInterface();
                }
            }catch(Exception ex)
            {
                MessageBox.Show("Error selecting device: "+ex.ToString());
            }
        }

        /// <summary>
        /// Show an error message
        /// </summary>
        /// <param name="msg"></param>
        public void Error(string msg)
        {
            MessageBox.Show(msg);
            DisplayWindow(true);
        }

        private void GenerateTriplets(object sender, EventArgs e)
        {
            supplicant.GenerateTriplet();

        }

    }
}