using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using SCLib;

namespace vpnsim.supplicant.SIMCommunication
{
    public class APDUCommand
    {
        private const byte CLA = 0xA0;
        public enum instruction : byte
        {
            select = 0xA4,
            status = 0xF2,
            readBinary = 0xB0,
            updateBinary = 0xD6,
            readRecord = 0xB2,
            updateRecord = 0xDC,
            seek = 0xA2,
            increase = 0x32,
            verifyCHV = 0x20,
            changeCHV = 0x24,
            disableCHV = 0x26,
            enableCHV = 0x28,
            unblockCHV = 0x2C,
            invalidate = 0x04,
            rehabilitate = 0x44,
            runGSMAlgorithm = 0x88,
            sleep = 0xFA,
            getResponse = 0xC0
        }
        public static CardCommand VerifyCHV(int chvNum, char[] pinCode)
        {
            byte[] DATA = new byte[8];
            DATA[0] = Convert.ToByte((char) pinCode[0]);
            DATA[1] = Convert.ToByte((char) pinCode[1]);
            DATA[2] = Convert.ToByte((char) pinCode[2]);
            DATA[3] = Convert.ToByte((char) pinCode[3]);
            for(int i =4;i<=7;i++) DATA[i] = 0xFF;


            CardCommand apdu = new CardCommand(CLA, (byte) instruction.verifyCHV, 0x00, (byte)chvNum, DATA, 8);
            return apdu;
        }

        public static CardCommand ReadRecord(int recNo, SIM.recordMode mode)
        {
            CardCommand apdu = new CardCommand(CLA, (byte)instruction.readRecord, (byte)recNo, (byte)mode, 8);
            return apdu;
        }
        public static CardCommand Select(SIM.DedicatedFile file)
        {
            byte[] fileIndex = { (byte)((short)file >> 8), (byte)((short)file & 0xFF) };
            CardCommand apdu = new CardCommand(CLA, (byte)instruction.select, 0x00, 0x00, fileIndex, 0x02);
            return apdu;
        }
        public static CardCommand Select(SIM.ElementaryFile file)
        {
            byte[] fileIndex = { (byte)((short)file >> 8), (byte)((short)file & 0xFF) };
            CardCommand apdu = new CardCommand(CLA, (byte)instruction.select, 0x00, 0x00, fileIndex, 0x02);
            return apdu;
        }
        public static CardCommand ReadBinary(byte offsetHigh, byte offsetLow, int length)
        {
            CardCommand apdu = new CardCommand(CLA, (byte)instruction.readBinary, offsetHigh, offsetLow,length);
            return apdu;
        }
        public static CardCommand RunGSMAlgorithm(byte[] rand)
        {
            CardCommand apdu = new CardCommand(CLA, (byte)instruction.runGSMAlgorithm, 0x00, 0x00, rand, 0x0A);
            return apdu;
        }

        public static CardCommand GetResponse(byte length)
        {
            CardCommand apdu = new CardCommand(CLA, (byte)instruction.getResponse, 0x00, 0x00, length);
            return apdu;
        }
    }
    public class APDUResponse
    {
        public byte SW1, SW2;
        public byte[] DATA;
        public byte[] responseData;
        public bool error = true;
        public APDUResponse(byte[] resData)
        {
            this.responseData = resData;
            SW1 = responseData[responseData.Length - 2];
            SW2 = responseData[responseData.Length - 1];
            //Changed
            DATA = new byte[responseData.Length-2];
            Array.Copy(responseData,DATA,responseData.Length-2);
            if(SW1== 0x90|SW1==0x9F)
            {
                error = false;   
            }
        }
        public new string ToString()
        {
            string description = "(" + SCUtil.FromByte(SW1) + ":" + SCUtil.FromByte(SW2) + ") - ";
            switch(SW1)
            {
                case 0x90:
                    description += "normal ending of the command";
                    break;
                case 0x9F:
                    description += "length '" + SCUtil.FromByte(SW2) + "' of the reponse data";
                    break;
                case 0x92:
                    switch(SW2)
                    {
                        case 0x40:
                            description += "memory problem";
                            break;
                        default:
                            description += "update successful but after using an internal retry routine " + SCUtil.FromByte(SW2) +
                                           " times";
                            break;
                    }
                    break;
                case 0x94:
                    switch(SW2)
                    {
                        case 0x00:
                            description += "no EF selected";
                            break;
                        case 0x02:
                            description += "out of range (invalid address)";
                            break;
                        case 0x04:
                            description += "file ID not found";
                            break;
                        case 0x08:
                            description += "file is inconsistent with the command";
                            break;
                    }
                    break;
                case 0x98:
                    switch(SW2)
                    {
                        case 0x02:
                            description += "no CHV initialised";
                            break;
                        case 0x04:
                            description += "access condition not fulfilled";
                            break;
                        case 0x08:
                            description += "in contradiction with CHV status";
                            break;
                        case 0x10:
                            description += "in contradiction with invalidation status";
                            break;
                        case 0x40:
                            description += "unsuccessful CHV verification, no attempt left";
                            break;
                        case 0x50:
                            description += "increase cannot be performed, Max value reached";
                            break;
                    }
                    break;
            }
            return description;
        }
        
    }
    
}
