using System;
using System.Collections.Generic;
using System.Net.Sockets;
using System.Text;
using System.Windows.Forms;
using InTheHand.Net;
using InTheHand.Net.Bluetooth;
using InTheHand.Net.Sockets;
using InTheHand.Windows.Forms;
using log4net;
using SCLib;
using Utility;
using gsm.datasource;

namespace vpnsim.supplicant.simSupplicant
{
    /// <summary>
    /// A Dummy Interface for testing the supplicant
    /// </summary>
    public class DummyInterface : SimInterface
    {
        private static readonly ILog log = LogManager.GetLogger(typeof(DummyInterface));
        private byte[] imsi = new byte[15];
        
        /// <summary>
        /// Constructor for the Dummy Interface. Generates a Dummy Interface with a random IMSI
        /// </summary>
        public DummyInterface(){
            Random rand = new Random();
            for(int i=0;i<imsi.Length;i++){
                imsi[i] = (byte)rand.Next(0, 9);
            }
            log.Debug("IMSI: " + HexEncoding.IMSIConvert(imsi));
        }

        /// <summary>
        /// Required by the interface, not really used
        /// </summary>
        /// <param name="deviceAddress"></param>
        public override void SetInterfaceName(string deviceAddress)
        {
        }
     
        /// <summary>
        /// Required by the interface, not really used
        /// </summary>
        /// <returns></returns>
        public override bool Connect()
        {
            return true;
        }

        /// <summary>
        /// Return status, always ready!
        /// </summary>
        /// <returns></returns>
        public override bool IsReady()
        {
            return true;
        }
        /// <summary>
        /// Run the GSM Algorithm
        /// </summary>
        /// <param name="rand">The RAND value</param>
        /// <returns>Whether the algorithm ran ok</returns>
        public override bool RunGSMAlgorithm(byte[] rand)
        {
            response = TransmitAPDU(ApduCommands.RunGSMAlgorithm(rand));
            if (response.error) return false;
            //response = TransmitAPDU(ApduCommands.GetResponse(response.SW2));
            return true;
        }

        /// <summary>
        /// Fake the sending of an APDU
        /// </summary>
        /// <param name="apdu"></param>
        /// <returns></returns>
        public override ApduResponse TransmitAPDU(CardCommand apdu)
        {
            
            ApduResponse res = null;
            switch((ApduCommands.instruction)apdu.INS)
            {
                case ApduCommands.instruction.readBinary:
                    byte[] imsiData = new byte[9+2];
                    imsiData[0] = 0x08;
                    imsiData[1] = (byte)(imsi[0] << 4 | 0x01);
                    int index = 1;
                    //Convert imsi to the format stored in the SIM
                    for (int i = 2; i < imsiData.Length - 2; i++)
                    {
                        imsiData[i] = (byte)(imsi[index++] | imsi[index++] << 4);
                    }
                    imsiData[imsiData.Length - 2] = 0x90;
                    imsiData[imsiData.Length - 1] = 0x00;

                    res = new ApduResponse(imsiData);
                    break;
                case ApduCommands.instruction.runGSMAlgorithm://Rungsmalg

                    TripletSource ts = new TripletSource();
                    byte[] rand = apdu.GetData();
                    byte[] sres, kc;

                    //Convert IMSI to normal format

                    bool exists = ts.GetSRES(imsi, rand,out sres, out kc);
                    byte[] data = new byte[16]; //Length of result
                    if (exists)
                    {
                        Array.Copy(sres, 0, data, 0, 4);
                        Array.Copy(kc, 0, data, sres.Length, 8);

                    }
                    else
                    {
                        Random random = new Random();
                        random.NextBytes(data);
                        Array.Resize(ref data, data.Length + 2);
                    }
                    data[data.Length - 2] = 0x90;
                    data[data.Length - 1] = 0x00;
                    res = new ApduResponse(data);
                    break;
                default:
                    res = new ApduResponse(new byte[] { 0x90, 0x00 });
                    break;
            }
            return res;
        }

        /// <summary>
        /// Required by the interface, returns true
        /// </summary>
        /// <returns></returns>
        public override bool Disconnect()
        {
            return true;
        }

        /// <summary>
        /// Required by the interface, not implemented
        /// </summary>
        /// <returns></returns>
        public override bool Reset()
        {
            throw new NotImplementedException();
        }
    }
}
